package com.ruoyi.tool.oss;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

/**
 * 阿里云存储
 *
 * @author Mark sunlightcs@gmail.com
 */
public class AliOssService extends CloudStorageService {
  private OSS client;

  public AliOssService(CloudStorageConfig config) {
    this.config = config;
    // 初始化
    init();
  }

  private void init() {
    String transEndpoint = String.format(config.getAliyunEndPoint(), "");
    client = new OSSClientBuilder().build(transEndpoint, config.getAliyunAccessKeyId(),
        config.getAliyunAccessKeySecret());
  }

  @Override
  public OssResObj upload(byte[] data, String path) {
    return upload(new ByteArrayInputStream(data), path);
  }

  @Override
  public OssResObj upload(InputStream inputStream, String path) {
    try {
      client.putObject(config.getAliyunBucketName(), path, inputStream);
    } catch (Exception e) {
      throw new RuntimeException("上传文件失败，请检查配置信息", e);
    }
    OssResObj resObj = new OssResObj();
    resObj.setDomain(config.getAliyunDomain());
    resObj.setFilePath("/" + path);
    return resObj;
  }

  // @Override
  // public OssResObj uploadSuffix(byte[] data, String suffix) {
  // return upload(data, getPath(config.getAliyunPrefix(), suffix));
  // }

  @Override
  public OssResObj uploadSuffix(byte[] data, String prefix, String suffix) {
    return upload(data, getPath(prefix, suffix));
  }

  // @Override
  // public OssResObj uploadSuffix(InputStream inputStream, String suffix) {
  // return upload(inputStream, getPath(config.getAliyunPrefix(), suffix));
  // }
}
