package com.ruoyi.tool.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.tool.mapper.StoreFileMapper;
import com.ruoyi.tool.domain.StoreFile;
import com.ruoyi.tool.service.IStoreFileService;
import org.springframework.transaction.annotation.Transactional;
import org.apache.commons.collections4.ListUtils;

/**
 * 文件Service业务层处理
 * 
 * @author rongxin
 * @date 2023-09-06
 */
@Service
public class StoreFileServiceImpl implements IStoreFileService 
{
    @Autowired
    private StoreFileMapper storeFileMapper;

    /**
     * 查询文件
     * 
     * @param id 文件主键
     * @return 文件
     */
    @Override
    public StoreFile selectStoreFileById(Long id)
    {
        return storeFileMapper.selectStoreFileById(id);
    }

    /**
     * 查询文件列表
     * 
     * @param storeFile 文件
     * @return 文件
     */
    @Override
    public List<StoreFile> selectStoreFileList(StoreFile storeFile)
    {
        return storeFileMapper.selectStoreFileList(storeFile);
    }

    /**
     * 新增文件
     * 
     * @param storeFile 文件
     * @return 结果
     */
    @Override
    public int insertStoreFile(StoreFile storeFile)
    {
        storeFile.setCreateTime(DateUtils.getNowDate());
        return storeFileMapper.insertStoreFile(storeFile);
    }

    /**
     * 批量新增文件
     *
     * @param list 文件
     * @return 结果
     */
    @Override
    @Transactional
    public int insertStoreFileBatch(List<StoreFile> list){
        List<List<StoreFile>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
                storeFileMapper.insertStoreFileBatch(splist);
        });
        return 1;
    }

    /**
     * 修改文件
     * 
     * @param storeFile 文件
     * @return 结果
     */
    @Override
    public int updateStoreFile(StoreFile storeFile)
    {
        storeFile.setUpdateTime(DateUtils.getNowDate());
        return storeFileMapper.updateStoreFile(storeFile);
    }

    /**
     * 批量修改 文件
     *
     * @param list 文件
     * @return 结果
     */
    @Override
    @Transactional
    public int updateStoreFileBatch(List<StoreFile> list) {
        List<List<StoreFile>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
                storeFileMapper.updateStoreFileBatch(splist);
        });
        return 1;
    }

    /**
     * 批量删除文件
     * 
     * @param ids 需要删除的文件主键
     * @return 结果
     */
    @Override
    public int deleteStoreFileByIds(Long[] ids)
    {
        return storeFileMapper.deleteStoreFileByIds(ids);
    }

    /**
     * 删除文件信息
     * 
     * @param id 文件主键
     * @return 结果
     */
    @Override
    public int deleteStoreFileById(Long id)
    {
        return storeFileMapper.deleteStoreFileById(id);
    }

    /**
     * 单条条件查询文件
     *
     * @param storeFile 文件
     * @return 文件条目
     */
    @Override
    public StoreFile selectStoreFile(StoreFile storeFile) {
        return storeFileMapper.selectStoreFile(storeFile);
    }

    /**
     * 条件查询文件数量
     *
     * @param storeFile 文件
     * @return 文件数量
     */
    @Override
    public Long selectStoreFileCount(StoreFile storeFile) {
        return storeFileMapper.selectStoreFileCount(storeFile);
    }

    /**
     * 条件查询文件是否存在
     *
     * @param storeFile 文件
     * @return 文件是否存在
     */
    @Override
    public boolean selectStoreFileExists(StoreFile storeFile) {
        return storeFileMapper.selectStoreFileExists(storeFile) > 0;
    }
}
